<?php
/**
* 2007-2020 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2015 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_'))
    exit();

class Affilbox extends Module
{

    public function __construct()
    {
        $this->name = 'affilbox';
        $this->tab = 'front_office_features';
        $this->version = '1.0.2';
        $this->author = 'PixelBrothers.cz';
        $this->author_email = 'info@pixelbrothers.cz';
        $this->author_website = 'https://www.pixelbrothers.cz';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = array('min' => '1.6.1.0', 'max' => _PS_VERSION_);
        $this->bootstrap = true;
        $this->updateUrl = 'https://download.affilbox.cz/prestashop/';

        parent::__construct();

        $this->displayName = $this->l('AffilBox', 'affilbox');
        $this->description = $this->l('Adds Affilbox tracking code to each page and conversion code to confirmation page.', 'affilbox');

        $this->confirmUninstall = $this->l('Skutečně chcete modul odinstalovat?', 'affilbox');
    }

    public function checkToken($json)
    {
        return ($json->token == Configuration::get($this->name.'_token') ? 200 : 401);
    }

    public function saveCodes($json)
    {
        if ($json->token == Configuration::get($this->name.'_token'))
        {
            $conversionCodeRaw = '{literal}' . $json->conversionCode . '{/literal}';
            $trackingCodeRaw = '{literal}' . $json->trackingCode . '{/literal}';
            // NOTE: Convert string into TPL
            $conversionCode = str_replace(
                array(
                    'COST',
                    'TRANSACTION_ID',
                    'CURRENCY'
                ),
                array(
                    '{/literal}{$affilbox_total_products}{literal}',
                    '{/literal}{$affilbox_reference}{literal}',
                    '{/literal}{$affilbox_currency}{literal}'
                ),
                $conversionCodeRaw
            );
            $trackingCode = $trackingCodeRaw;
            // NOTE: Save values into DB
            Configuration::updateValue($this->name.'_trackingcode', $trackingCode, true);
            Configuration::updateValue($this->name.'_conversioncode', $conversionCode, true);
            // NOTE: Overwrite TPL files
            $c = fopen(dirname(__FILE__) . '/views/templates/hook/conv.tpl', 'w');
            fwrite($c, $conversionCode);
            fclose($c);
            $t = fopen(dirname(__FILE__) . '/views/templates/hook/tracking.tpl', 'w');
            fwrite($t, $trackingCode);
            fclose($t);
        }
    }

    public function generateToken()
    {
        $token = md5($_SERVER['SERVER_NAME'] . date('Y-md-d H:i:s'));
        return $token;
    }

    public function install()
    {
        if (version_compare(_PS_VERSION_, "1.6.99.99", ">"))
        {
            if (!parent::install() ||
                !$this->registerHook('displayOrderConfirmation') ||
                !$this->registerHook('displayHeader')) return false;
        }
        else
        {
            if (!parent::install() ||
                !$this->registerHook('displayOrderConfirmation') ||
                !$this->registerHook('header')) return false;
        }

        Configuration::updateValue($this->name.'_trackingcode', '');
        Configuration::updateValue($this->name.'_conversioncode', '');
        Configuration::updateValue($this->name.'_token', $this->generateToken());
        Configuration::updateValue($this->name.'_configuration', '');

        return true;
    }

    public function uninstall()
    {
        return parent::uninstall()
        && Configuration::deleteByName($this->name.'_trackingcode')
        && Configuration::deleteByName($this->name.'_conversioncode')
        && Configuration::deleteByName($this->name.'_configuration')
        && Configuration::updateValue($this->name.'_token', '');
    }

    // NOTE: Order confirmation Hook for both versions
    public function hookDisplayOrderConfirmation($params)
    {
        $id_currency = ($params['order']->id_currency ? $params['order']->id_currency : $params['objOrder']->id_currency);
        $currency = new Currency($id_currency);

        $this->context->smarty->assign(
            // array('affilbox_total_products' => ((float) $params['order']->total_products_wt ? (float) $params['order']->total_products_wt : (float) $params['objOrder']->total_products_wt),
            //       'affilbox_reference' => ($params['order']->reference ? $params['order']->reference : $params['objOrder']->reference))
            array(
                'affilbox_total_products' => (
                    Configuration::get($this->name.'_configuration') == 'total_wt' ?
                    ((float) $params['order']->total_products_wt ? ((float) $params['order']->total_products_wt + (float) $params['order']->total_shipping_tax_incl) : ((float) $params['objOrder']->total_products_wt + (float) $params['objOrder']->total_shipping_tax_incl)) :
                    ((float) $params['order']->total_products_wt ? (float) $params['order']->total_products : (float) $params['objOrder']->total_products)
                ),
                'affilbox_reference' => (
                    $params['order']->reference ? $params['order']->reference : $params['objOrder']->reference
                ),
                '$affilbox_currency' => $currency->iso_code
            )
        );
        return $this->display(__FILE__, 'conv.tpl');
    }

    // NOTE: Header Hook for PS 1.7
    public function hookDisplayHeader()
    {
        return $this->display(__FILE__, 'tracking.tpl');
    }

    // NOTE: Header Hook for PS 1.6
    public function hookHeader()
    {
        return $this->display(__FILE__, 'tracking.tpl');
    }

    public function check4Updates($currentVersion)
    {
        $newVersion = Tools::file_get_contents($this->updateUrl . 'version.txt');
        return (version_compare(trim(str_replace('AvailableVersion: ', '', $newVersion)), $currentVersion, '>') ? 'available' : 'actual');
    }

    /*********************************/
    /*         ADMIN CONTENT         */
    /*********************************/

    public function getContent()
    {
        if (Tools::isSubmit('submitConfiguratorGeneralSettings')) {
            $before .= $this->processGeneralSettingsSubmit();
        }

        // $updateAvailability = $this->check4Updates($this->version);
        $this->context->smarty->assign(
            array('token' => Configuration::get($this->name.'_token'),
                  'conversion' => (Configuration::get($this->name.'_conversioncode') ? 1 : 0),
                  'tracking' => (Configuration::get($this->name.'_trackingcode') ? 1 : 0),
                  'configuration' => Configuration::get($this->name.'_configuration'),
                  'form' => $this->displayGeneralSettingsForm(),
                  'upgrade' => $this->check4Updates($this->version))
        );
        return $this->display(dirname(__FILE__), '/views/templates/admin/admin.tpl');
    }

    /*********************************/
    /*             FORMS             */
    /*********************************/

    // NOTE: generate form for general settings
    protected function displayGeneralSettingsForm()
    {
        $options = array(
            array(
                'type' => '',
                'label' => '-- vyberte --'
            ),
            array(
                'type' => 'total_wt',
                'label' => 'Objednávka s DPH + poštovné'
            ),
            array(
                'type' => 'total',
                'label' => 'Objednávka bez DPH - poštovné - sleva z kuponu'
            )
        );

        $fieldsForm = [];
        $fieldsForm[0]['form'] = [
            'legend' => [
                'title' => $this->l('Nastavení modulu'),
                'icon' => 'icon-puzzle-piece',
            ],
            'input' => [
                [
                    'type' => 'select',
                    'label' => $this->l('Celková částka objednávky = '),
                    'name' => $this->name . '_configuration',
                    'options' => [
                      'query' => $options,
                      'id' => 'type',
                      'name' => 'label',
                    ],
                    'required' => true
                ]
            ],
            'submit' => [
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right'
            ]
        ];

        $helper = new HelperForm();
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->allow_employee_form_lang = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submitConfiguratorGeneralSettings';
        $helper->fields_value = $helper->fields_value
            + Configuration::getMultiple([
                $this->name . '_configuration'
            ]);

        return $helper->generateForm($fieldsForm);
    }

    /*********************************/
    /*        PROCESS METHODS        */
    /*********************************/

    // NOTE: Process submitted form - ON or OFF
    protected function processGeneralSettingsSubmit()
    {
        $values = array_filter([
            $this->name . '_configuration' => Tools::getValue($this->name . '_configuration', null),
        ], function($item) {
            return $item !== null;
        });

        if (count($values)) {
            foreach ($values as $key => $value) {
                Configuration::updateValue($key, $value);
            }
            return $this->displayConfirmation($this->l('Nastavení upraveno'));
        } else {
            return $this->displayError($this->l('Invalid Configuration value'));
        }
    }

}
