<?php
/**
* 2007-2020 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2015 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

class AffilboxUpgrade
{

    public function __construct($module)
    {
        $this->currentVersion = $module->version;
        $this->updateUrl = $module->updateUrl;
        $this->module = $module;
    }

    public function getLatestVersion()
    {
        $newVersion = Tools::file_get_contents($this->updateUrl . 'version.txt');
        return str_replace('AvailableVersion: ', '', $newVersion);
    }

    public function copyFiles()
    {
        if (ini_get('allow_url_fopen')) {
            echo '<li class="success">I can handle urls</li>';
        } else {
            echo '<li class="info">I cannot handle urls. Trying to enable url handling...</li>';
            ini_set('allow_url_fopen', true);
            if (ini_get('allow_url_fopen')) {
                echo '<li class="success"> --- succeeded.</li>';
            } else {
                echo '<li class="error"> --- failed.</li>';
            }
        }

        $files = array('zip', 'txt');
        foreach ($files as $file)
        {
            $v = trim($this->getLatestVersion());
            $url = $this->updateUrl . $v . '/upgrade-' . $v . '.' . $file;
            $destination = ($file == 'txt' ? './upload/upgrade-' . $v . '.' . 'php' : './upload/' . basename($url));

            try {
            		$data = Tools::file_get_contents($url);
            		$handle = fopen($destination, "w");
            		fwrite($handle, $data);
            		fclose($handle);
                echo '<li class="success">File <em>' . basename($destination) . '</em> copied!</li>';
          	} catch (Exception $e) {
            		echo '<li class="error">Caught exception: ',  $e->getMessage(), "</li>";
                return false;
          	}
        }
        return true;
    }

    public function update()
    {
        include('./upload/upgrade-' . trim($this->getLatestVersion()) . '.php');
        echo '<li class="success">Update initialized</li>';
        echo '<li class="success">Starting update...</li>';
        $dynamicUpgradeClass = str_replace('.', '_', 'Upgrade_' . trim($this->getLatestVersion()));
        $update = new $dynamicUpgradeClass($this->module);
        foreach ($update->versionList as $ver)
        {
            echo (version_compare($ver, $this->currentVersion, '<=') ? '<li class="info">Skipping version ' . $ver . '</li>' : '<li class="success">Updating to ' . $ver . '</li>');
            if (version_compare($ver, $this->currentVersion, '>'))
            {
                $dynamicMethod = str_replace('.', '_', 'execute_' . trim($this->getLatestVersion()));
                $u = $update->$dynamicMethod();
                echo ($u ? '<li class="success">Updating to ' . $ver . ' successful</li>' : '<li class="error">Updating to ' . $ver . ' error</li>');
            }
        }
        $zip = new ZipArchive;
        $res = $zip->open('./upload/upgrade-' . trim($this->getLatestVersion()) . '.zip');
        if ($res === TRUE) {
            $zip->extractTo('./../../');
            $zip->close();
            echo '<li class="success">Extracting files successful</li>';
            if (unlink('./upload/upgrade-' . trim($this->getLatestVersion()) . '.zip') &&
                unlink('./upload/upgrade-' . trim($this->getLatestVersion()) . '.php'))
            {
                echo '<li class="success">Deleting autoupdate files successful</li>';
                if (Db::getInstance()->update('module', array(
                    'version' => trim($this->getLatestVersion())
                ), 'name = ' . $this->module->name, 1, true))
                {
                    echo '<li class="success">Update succesful</li>';
                }
                else
                {
                    echo '<li class="error">Error during update finish. Please report this issue.</li>';
                }
            }
            else
            {
                echo '<li class="error">Error during deleted autoupdate files</li>';
            }
        } else {
            echo '<li class="error">Extracting files error</li>';
        }
    }

    public function execute()
    {
        ini_set('display_errors', 1);
        ini_set('display_startup_errors', 1);
        error_reporting(E_ALL);
        if ($this->copyFiles())
        {
            echo '<li class="success">Files copied!</li>';
            $this->update();
        }
        else
        {
            echo '<li class="error">Files cannot be copied.</li>';
        }
    }

}
